#include <cstdio>
#include <cstring>

using namespace std;

/*
 *  This program reads a MIDI file and explains its contents.
 *
 *  - Copyright (C) 1992,2003 Bisqwit (http://iki.fi/bisqwit/)
 */

/* Useful typedefs. Extremely dangerous to modify. */
typedef unsigned long dword;
typedef unsigned short word;
typedef unsigned char byte; 

static dword ConvL(const byte *s)
{
    return (((dword)s[0] << 24) | ((dword)s[1] << 16) | ((word)s[2] << 8) | s[3]);
}
    
static word ConvI(const byte *s)
{    
    return (s[0] << 8) | s[1];
}
    
static dword GetVarLen(byte *track, size_t *Posi)
{
    dword d = 0;
    for(;;)
    {
        byte b = track[(*Posi)++];
        d = (d<<7) + (b&127);
        if(b < 128)break;
    }
    return d;
}
int main(void)
{
    byte Signa[4];
    unsigned tracks;
    
    fread(&Signa, 1, 4, stdin);
    printf("MIDI signature: '%.4s' (should be 'MThd')\n", Signa);
    fread(&Signa, 1, 4, stdin);
	printf("Header length: %u (should be 6)\n", (unsigned)ConvL(Signa));
    fread(&Signa, 1, 2, stdin); printf("Format (0,1,2): %u\n", (unsigned)ConvI(Signa));
    fread(&Signa, 1, 2, stdin); printf("Track count: %u\n", tracks=(unsigned)ConvI(Signa));
    fread(&Signa, 1, 2, stdin); printf("Deltaticks: %u\n", (unsigned)ConvI(Signa));
    
    for(unsigned a=0; a<tracks; ++a)
    {
	    unsigned trklen;
    	printf("---Track %u---\n", a);
	    fread(&Signa, 1, 4, stdin);
	    printf("Track signature: '%.4s' (should be 'MTrk')\n", Signa);
	    fread(&Signa, 1, 4, stdin);
		printf("Track length: %u bytes\n", trklen=ConvL(Signa));
		unsigned running=0, trkpos=0;
		while(trkpos<trklen)
		{
			unsigned delay=0, len=0, type=0;
			for(;;)
			{
				byte b = getchar(); ++trkpos;
				delay = (delay << 7) | (b & 127);
				if(b < 128)break;
			}
			printf("<%7u> ", delay);
			
			byte b = getchar();
			if(b & 128)
			{
				if(b < 0xF0)running = b;
				++trkpos;
				printf(" %02X ", b);
			}
			else
			{
				ungetc(b, stdin);
				b = running;
				printf("[%02X]", b);
			}
			if(b == 0xFF)
			{
				printf("%02X ", type=getchar()); ++trkpos;
				printf("%02X ", len=getchar()); ++trkpos;
				
DumpString:
				char *Buf = new char[len];
				fread(Buf, 1, len, stdin); trkpos += len;
				
				if(type == 0x51) // tempo
				{
        			for(unsigned a=type=0; a<len;)
        			{
        				byte b = Buf[a++];
        				type<<=8; type+=b;
        			}
        			printf("<%u>", type);
				}
				else
				{
    				for(unsigned a=0; a<len; ++a)
    				{
    					b = Buf[a];
    					if(b=='\\')printf("\\\\");
    					else if(b<32 || b>=127)printf("\\x%02X", b);
    					else putchar(b);
    				}
				}
				delete[] Buf;
				printf("\n");
				continue;
			}
			if(b == 0xF0 || b == 0xF7)
			{
				type=0;
    			for(len=0;;)
    			{
    				byte b = getchar(); ++trkpos;
    				len = (len << 7) | (b & 127);
    				if(b < 128)break;
    			}
    			printf("<%u> ", len);
    			goto DumpString;
			}
			len = 1;
			if(b >= 0xC0 && b <= 0xDF) len = 1;
			else if(b >= 0x80 && b <= 0xEF) len = 2;
			else if(b == 0xF2) len = 2;
			while(len > 0)
			{
				b = getchar(); --len; ++trkpos;
				printf("%02X ", b);
			}
			printf("\n");
		}
		if(trkpos > trklen) printf("-- Track longer than expected!\n");
    }
    printf("--File size: %ld--\n", (long)ftell(stdin));
    return 0;
}
