#include <string.h>

#include "core.hh"

#include "snes9x.h"
#include "soundux.h"
#include "libspc.h"

static SPC_Config cfg = { 44100,16, 2, 1, 1 };

void SPCCore_MuteSound() { S9xSetSoundMute(TRUE); }
void SPCCore_SetChannels(int n) { cfg.channels = n; }
void SPCCore_SetBitness(int n) { cfg.resolution = n; }
void SPCCore_SetInterpolation(int n) { cfg.is_interpolation = n; }
void SPCCore_SetEcho(int n) { cfg.is_echo = n; Settings.DisableSoundEcho=n; }
void SPCCore_SetAlt(int n) { Settings.AltSampleDecode = n; }
void SPCCore_SetRate(int n) { cfg.sampling_rate = n; }
void SPCCore_SetPlaybackSpeed(int n) { cur_rate = n; }
void SPCCore_SetChannelStatus(int n, int s)
{
    if(s)
        so.sound_switch |= (1<<n);
    else
        so.sound_switch &= ~(1<<n);
}

int SPCCore_GetAlt() { return Settings.AltSampleDecode; }
int SPCCore_GetEcho() { return Settings.DisableSoundEcho; }
int SPCCore_GetRate() { return cfg.sampling_rate; }
int SPCCore_GetChannels() { return cfg.channels; }
int SPCCore_GetBitness() { return cfg.resolution; }
int SPCCore_GetChannelStatus(int n) { return so.sound_switch&(1<<n); }
int SPCCore_GetPlaybackSpeed() { return cur_rate; }
int SPCCore_GetInterpolation() { return cfg.is_interpolation; }
int SPCCore_GetNumInterpolation() { return 2; }

void SPCCore_Terminate() { SPC_close(); }

unsigned SPCCore_Init()
{
#ifdef DEBUGGER
    Settings.TraceSoundDSP = 1; 
#endif
    Settings.FixFrequency = 1;
    return SPC_init(&cfg);
}
int SPCCore_Load(const char* fn) { return !SPC_load(fn) ? -1 : 0; }

void SPCCore_Generate(void* buffer)
{
    SPC_update((unsigned char*) buffer);
}

SPCCore_ID* SPCCore_GetID(const char *fn)
{
    SPC_ID666* tmp = SPC_get_id666(fn);
    if(!tmp) return NULL;
    
    static SPCCore_ID stat;
    memset(&stat, 0, sizeof(stat));
    
    strncpy(stat.songname,  tmp->songname,  33);
    strncpy(stat.gametitle, tmp->gametitle, 33);
    strncpy(stat.dumper,    tmp->dumper,    17);
    strncpy(stat.comments,  tmp->comments,  33);
    strncpy(stat.author,    tmp->author,    33);
    stat.playtime = tmp->playtime;
    stat.fadetime = tmp->fadetime;
    stat.emulator = tmp->emulator;
    
    return &stat;
}
